@@ -8,8 +8,8 @@ from ultralytics.yolo.utils.metrics import ClassifyMetrics
 
 class ClassificationValidator(BaseValidator):
 
-    def __init__(self, dataloader=None, save_dir=None, pbar=None, args=None):
-        super().__init__(dataloader, save_dir, pbar, args)
+    def __init__(self, dataloader=None, save_dir=None, pbar=None, args=None, _callbacks=None):
+        super().__init__(dataloader, save_dir, pbar, args, _callbacks)
         self.args.task = 'classify'
         self.metrics = ClassifyMetrics()
 
@@ -43,6 +43,8 @@ class ClassificationValidator(BaseValidator):
         return build_classification_dataloader(path=dataset_path,
                                                imgsz=self.args.imgsz,
                                                batch_size=batch_size,
+                                               augment=False,
+                                               shuffle=False,
                                                workers=self.args.workers)
 
     def print_results(self):
